/*
 * MIT License
 *
 * Copyright (c) 2020 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package earlystop

import (
	"log"

	neuralnetwork "git.semlanik.org/semlanik/NeuralNetwork/neuralnetwork"
	training "git.semlanik.org/semlanik/NeuralNetwork/training"
)

type constantRateEarlyStop struct {
	network     *neuralnetwork.NeuralNetwork
	trainer     training.Trainer
	minFailRate float64
}

func NewConstantRateEarlyStop(minFailRate float64, network *neuralnetwork.NeuralNetwork, trainer training.Trainer) (es *constantRateEarlyStop) {
	es = nil
	if network == nil || trainer == nil {
		return
	}

	es = &constantRateEarlyStop{
		network:     network,
		trainer:     trainer,
		minFailRate: minFailRate,
	}
	return
}

func (es *constantRateEarlyStop) Test() bool {
	_, fails, total := es.network.Validate(es.trainer)
	log.Printf("Fail count: %v/%v\n", fails, total)

	failRate := float64(fails) / float64(total)
	return es.minFailRate >= failRate
}

func (es *constantRateEarlyStop) Reset() {
}
