/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package neuralnetwork

import (
	training "git.semlanik.org/semlanik/NeuralNetwork/training"
	mat "gonum.org/v1/gonum/mat"
)

const (
	StateIdle       = 1
	StateLearning   = 2
	StateValidation = 3
	StatePredict    = 4
)

type SubscriptionFeatures uint8

const (
	ActivationsSubscription SubscriptionFeatures = 1 << iota
	BiasesSubscription
	WeightsSubscription
	TrainingSubscription
	ValidationSubscription
	StateSubscription
	AllSubscription = 0xFF
)

type StateWatcher interface {
	Init(nn *NeuralNetwork)
	UpdateState(state int)
	UpdateActivations(l int, a *mat.Dense)
	UpdateBiases(l int, biases *mat.Dense)
	UpdateWeights(l int, weights *mat.Dense)
	UpdateTraining(t int, epocs int, samplesProcced int, totalSamplesCount int)
	UpdateValidation(validatorCount int, failCount int)
	GetSubscriptionFeatures() SubscriptionFeatures
}

func (f SubscriptionFeatures) Has(flag SubscriptionFeatures) bool {
	return f&flag != 0
}

func (f *SubscriptionFeatures) Set(flag SubscriptionFeatures) {
	*f |= flag
}

func (f *SubscriptionFeatures) Unset(flag SubscriptionFeatures) {
	*f &= (^flag)
}

func (f *SubscriptionFeatures) Clear() {
	*f = 0
}

type BatchWorker interface {
	Run(trainer training.Trainer, startIndex, endIndex int)
	Result(layer int) (dB, dW *mat.Dense, batchSize int)
}

type BatchWorkerFactory interface {
	GetBatchWorker() BatchWorker
	GetAvailableThreads() int
}

type EarlyStop interface {
	Test() bool
	Reset()
}
